/***************** LCD interface *****************
* This code will interface to a standard LCD controller
* like the Hitachi HD44780. It uses LCD display in 4 bit mode, with
* the hardware connected as follows:
* PORTD bits 4-7 are connected to the LCD data bits 4-7 (high nibble)
* PORTB bit 0 is connected to the LCD RS input (register select)
* PORTB bit 1 is connected to the LCD EN bit (enable)
**************************************************/

#define LCD_RS PORTB.0    // Register select
#define LCD_EN PORTB.1    // Enable
#define LCD_D4 PORTD.4    // Data bits
#define LCD_D5 PORTD.5    // Data bits
#define LCD_D6 PORTD.6    // Data bits
#define LCD_D7 PORTD.7    // Data bits

// Strob LCD
void LCD_STROBE(void)
  {
    LCD_EN = 1;
    delay_us(5);
    LCD_EN = 0;
    delay_us(40);
  }
  
// write a byte to the LCD in 4 bit mode
void lcd_write(unsigned char c)
  {
    if(c & 0x80) LCD_D7=1; else LCD_D7=0;
    if(c & 0x40) LCD_D6=1; else LCD_D6=0;
    if(c & 0x20) LCD_D5=1; else LCD_D5=0;
    if(c & 0x10) LCD_D4=1; else LCD_D4=0;
    LCD_STROBE();
    if(c & 0x08) LCD_D7=1; else LCD_D7=0;
    if(c & 0x04) LCD_D6=1; else LCD_D6=0;
    if(c & 0x02) LCD_D5=1; else LCD_D5=0;
    if(c & 0x01) LCD_D4=1; else LCD_D4=0;
    LCD_STROBE();
  }

// Clear and home the LCD
void lcd_clear(void)
  {
    LCD_RS = 0;
    lcd_write(0x1);
    delay_ms(50);
  }

// Write a string of chars to the LCD
void lcd_puts(const char *s)
  {
    LCD_RS = 1;    // write characters
    while(*s) lcd_write(*s++);
  }

// write one character to the LCD
void lcd_putch(unsigned char c)
  {
    LCD_RS = 1;    // write characters
    lcd_write(c);
  }

// Go to the specified position
void lcd_goto(unsigned char pos, unsigned char line)
  {
    LCD_RS = 0;
    if (line==0)
      lcd_write(0x80 + pos);
    else
      lcd_write(0x80 + pos + 0x40);
  }

// initialise the LCD - put into 4 bit mode
void lcd_init(void)
  {
    LCD_RS = 0;    // write control bytes
    delay_ms(200); // power on delay
    LCD_D4 = 1;    // init!    
    LCD_D5 = 1;    //
    LCD_D6 = 0;    //
    LCD_D7 = 0;    //
    LCD_STROBE();
    delay_ms(40);
    LCD_STROBE();  // init!    
    delay_ms(40);
    LCD_STROBE();  // init!    
    delay_ms(40);
    LCD_D4 = 0;    // set 4 bit mode
    LCD_STROBE();    
    lcd_write(0x28); // 4 bit mode, 1/16 duty, 5x8 font, 2lines
    lcd_write(0x08);
    lcd_write(0x01); // clear display and reset cursor
    delay_ms(50);
    lcd_write(0x06); // entry mode advance cursor
    lcd_write(0x0C); // display on
  }

// Create simbol "mikro"

void LCD_build(unsigned char *ptr)
  {
    unsigned char i;

    LCD_RS = 0;
    lcd_write(0x40);      // point to CGRAM
    LCD_RS = 1;
    for(i=0; i<8; i++)
      lcd_write(ptr[i]);
  }

// Clear  line
void lcd_spc(unsigned char line)
  {
    lcd_goto(0,line);
    lcd_puts("                ");
  }

// Write Freq and Lim to display
void lcd_frq(unsigned char Nd, unsigned int AdcR)
  {
    unsigned char buf[6];
    lcd_spc(1);
    lcd_goto(0,1);
    lcd_puts("F=");
    if (Nd == 0) lcd_puts("1MHz   ");
    if (Nd == 1) lcd_puts("250KHz ");
    if (Nd == 2) lcd_puts("62KHz  ");
    if (Nd == 3) lcd_puts("16KHz  ");
    if (Nd == 4) lcd_puts("4KHz   ");
    if (Nd == 5) lcd_puts("1KHz   ");
    if (Nd == 6) lcd_puts("250Hz  ");
    if (Nd == 7) lcd_puts("62Hz   ");
    if (Nd == 8) lcd_puts("15Hz   ");
    lcd_puts("ADC=");
    if (AdcR > 999)
       AdcR = 999;
    itoa(AdcR, buf);
    lcd_puts(buf);
  }

// Leave only 3 significant digits
unsigned char prpeLC(unsigned char *string)
  {
    unsigned char len;

    len = strlen(string);      // length of string
    if (len > 3)
      {
        if (string[3] >= 0x35)
         {
            string[2] = string[2] + 1;
            if (string[2] > 0x39)                    // "9"
               {
                  string[2] = 0x30;                  // "0"
                  string[1] = string[1] + 1;
                  if (string[1] > 0x39)              // "9"
                     {
                        string[1] = 0x30;            // "0"
                        string[0] = string[0] + 1;
                     }
               }
         }
      }
    return len;  
  }

// output string to indicator
void outIND(unsigned char *string0, unsigned char len, unsigned char znak)
  {
    lcd_goto(0,0);
    if (pin_LC == 0)
        lcd_puts("C ");            // Mode "C"
    else  lcd_puts("L ");          // Mode "L"

    if (len == 8)
      {
        lcd_putch(string0[0]);
        lcd_putch('.');
        lcd_putch(string0[1]);
        lcd_putch(string0[2]);
        lcd_puts("0.000,0");
      }
    if (len < 8)
        lcd_puts("  ");

    if (len == 7)
      {
        lcd_putch(string0[0]);
        lcd_putch(string0[1]);
        lcd_putch(string0[2]);
        lcd_puts(".000,0");
      }
    if (len < 7)
        lcd_putch(0x20);

    if (len == 6)
      {
        lcd_putch(string0[0]);
        lcd_putch(string0[1]);
        lcd_putch('.');
        lcd_putch(string0[2]);
        lcd_puts("00,0");
      }
    if (len < 6)
        lcd_putch(0x20);

    if (len == 5)
      {
        lcd_putch(string0[0]);
        lcd_putch('.');
        lcd_putch(string0[1]);
        lcd_putch(string0[2]);
        lcd_puts("0,0");
      }
    if (len < 5)
        lcd_puts("  ");

    if (len == 4)
      {
        lcd_putch(string0[0]);
        lcd_putch(string0[1]);
        lcd_putch(string0[2]);
        lcd_puts(",0");
      }

    if (len == 3)
      {
        if (znak)
          lcd_putch('-');
        else
          lcd_putch(0x20);
        lcd_putch(string0[0]);
        lcd_putch(string0[1]);
        lcd_putch(',');
        lcd_putch(string0[2]);
      }

    if (len < 3)
      {
        if (znak)
          lcd_putch('-');
        else
          lcd_putch(0x20);
      }

    if (len == 2)
      {
        lcd_putch(string0[0]);
        lcd_putch(',');
        lcd_putch(string0[1]);
      }

    if (len == 1)
      {
        lcd_puts("0,");
        lcd_putch(string0[0]);
      }

    if (pin_LC == 0)
        lcd_puts("pF");       // Mode "C"
    else
         {
           lcd_putch(0x00);   // Mode "L"
           lcd_putch('H');
         }
    lcd_putch(0x20);
  }

/******************** EEPROM **************************/
// EEPROM write
void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
  {
    while(EECR & (1<<EEPE)); // Wait for completion of previous write
    EEAR = uiAddress;        // Set up address and Data Registers
    EEDR = ucData;           // ---------------------------------
    EECR |= (1<<EEMPE);      // Write logical one to EEMPE
    EECR |= (1<<EEPE);       // Start eeprom write by setting EEPE
  }

// EEPROM read
unsigned char EEPROM_read(unsigned int uiAddress)
  {
    while(EECR & (1<<EEPE)); // Wait for completion of previous write
    EEAR = uiAddress;        // Set up address register
    EECR |= (1<<EERE);       // Start eeprom read by writing EERE
    return EEDR;             // Return data from Data Register
  }

